#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
##    Copyright (C) 2007 Khertan khertan@khertan.net
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

PPKG_SECTIONS = "user/desktop, user/development, user/education, user/games," \
    " user/graphics, user/multimedia, user/navigation," \
    " user/network," \
    " user/office, user/science, user/system, user/utilities," \
    " accessories, communication, games, multimedia, office," \
    " other, programming, support, themes, tools".split(", ")

PPKG_ARCHS = "all any armel i386 ia64 alpha amd64 armeb arm hppa m32r" \
             " m68k mips mipsel powerpc ppc64 s390 s390x sh3 sh3eb sh4" \
             " sh4eb sparc darwin-i386 darwin-ia64 darwin-alpha darwin-amd64" \
             " darwin-armeb darwin-arm darwin-hppa darwin-m32r darwin-m68k" \
             " darwin-mips darwin-mipsel darwin-powerpc darwin-ppc64" \
             " darwin-s390 darwin-s390x darwin-sh3 darwin-sh3eb darwin-sh4" \
             " darwin-sh4eb darwin-sparc freebsd-i386 freebsd-ia64" \
             " freebsd-alpha freebsd-amd64 freebsd-armeb freebsd-arm" \
             " freebsd-hppa freebsd-m32r freebsd-m68k freebsd-mips" \
             " freebsd-mipsel freebsd-powerpc freebsd-ppc64 freebsd-s390" \
             " freebsd-s390x freebsd-sh3 freebsd-sh3eb freebsd-sh4" \
             " freebsd-sh4eb freebsd-sparc kfreebsd-i386 kfreebsd-ia64" \
             " kfreebsd-alpha kfreebsd-amd64 kfreebsd-armeb" \
             " kfreebsd-arm kfreebsd-hppa kfreebsd-m32r kfreebsd-m68k" \
             " kfreebsd-mips kfreebsd-mipsel kfreebsd-powerpc" \
             " kfreebsd-ppc64 kfreebsd-s390 kfreebsd-s390x kfreebsd-sh3" \
             " kfreebsd-sh3eb kfreebsd-sh4 kfreebsd-sh4eb kfreebsd-sparc" \
             " knetbsd-i386 knetbsd-ia64 knetbsd-alpha knetbsd-amd64" \
             " knetbsd-armeb knetbsd-arm knetbsd-hppa knetbsd-m32r" \
             " knetbsd-m68k knetbsd-mips knetbsd-mipsel knetbsd-powerpc" \
             " knetbsd-ppc64 knetbsd-s390 knetbsd-s390x knetbsd-sh3" \
             " knetbsd-sh3eb knetbsd-sh4 knetbsd-sh4eb knetbsd-sparc" \
             " netbsd-i386 netbsd-ia64 netbsd-alpha netbsd-amd64" \
             " netbsd-armeb netbsd-arm netbsd-hppa netbsd-m32r" \
             " netbsd-m68k netbsd-mips netbsd-mipsel netbsd-powerpc" \
             " netbsd-ppc64 netbsd-s390 netbsd-s390x netbsd-sh3" \
             " netbsd-sh3eb netbsd-sh4 netbsd-sh4eb netbsd-sparc" \
             " openbsd-i386 openbsd-ia64 openbsd-alpha" \
             " openbsd-amd64 openbsd-armeb openbsd-arm" \
             " openbsd-hppa openbsd-m32r openbsd-m68k openbsd-mips" \
             " openbsd-mipsel openbsd-powerpc openbsd-ppc64" \
             " openbsd-s390 openbsd-s390x openbsd-sh3 openbsd-sh3eb" \
             " openbsd-sh4 openbsd-sh4eb openbsd-sparc hurd-i386" \
             " hurd-ia64 hurd-alpha hurd-amd64 hurd-armeb hurd-arm" \
             " hurd-hppa hurd-m32r hurd-m68k hurd-mips" \
             " hurd-mipsel hurd-powerpc hurd-ppc64 hurd-s390" \
             " hurd-s390x hurd-sh3 hurd-sh3eb hurd-sh4" \
             " hurd-sh4eb hurd-sparc".split(" ")

PPKG_LICENSES = ["gpl", "lgpl", "bsd", "artistic", "shareware"]

PPKG_RETURN = """
----------------------------------------------------------------------
NAME        : %(name)s
----------------------------------------------------------------------
LICENSE     : %(license)s
URL         : %(url)s
AUTHOR      : %(author)s
MAIL        : %(email)s
----------------------------------------------------------------------
DEPENDS     : %(depends)s
ARCH        : %(arch)s
SECTION     : %(section)s
----------------------------------------------------------------------
DESCRIPTION :
%(description)s
----------------------------------------------------------------------
SCRIPTS : %(scripts)s
----------------------------------------------------------------------
FILES :
%(files)s
"""

PPKG_DEBIAN_CHANGESLOG =  \
    """%(name)s (%(version)s-%(buildversion)s) stable; urgency=low

  %(changelog)s

 -- %(author)s <%(email)s>  %(buildDate)s
  """

PPKG_DEBIAN_CONTROL = """Source: %(name)s
Section: %(section)s
Priority: optional
Maintainer: %(maintainer)s <%(email)s>
Build-Depends: %(build_depends)s
Standards-Version: 3.9.2

Package: %(name)s
XB-Maemo-Display-Name: %(display_name)s
Architecture: %(arch)s
Depends: %(depends)s
Suggests: %(suggests)s
Description: %(description)s
XB-Maemo-Upgrade-Description: %(upgrade_description)s
XB-Maemo-Flags: %(maemo_flags)s
XB-MeeGo-Desktop-Entry-Filename: %(meego_desktop_entry_filename)s
%(bugtrackerstr)s
%(iconstr)s"""

PPKG_LICENSES_TEXT = {
    "gpl": """
This package is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this package; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

On Debian systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
""",
    "lgpl": """
This package is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this package; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

On Debian systems, the complete text of the GNU Lesser General
Public License can be found in `/usr/share/common-licenses/LGPL'.
""",
    "bsd": """
Redistribution and use in source and binary forms, with or without
modification, are permitted under the terms of the BSD License.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

On Debian systems, the complete text of the BSD License can be
found in `/usr/share/common-licenses/BSD'.
""",
    "shareware": """
This product is copyrighted shareware, not public-domain software.
You may use the unregistered version at no charge for an evaluation period.
To continue to use the software beyond evaluation period, you must register it.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
""",
    "artistic": """
This program is free software; you can redistribute it and/or modify it
under the terms of the "Artistic License" which comes with Debian.

THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES
OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

On Debian systems, the complete text of the Artistic License
can be found in `/usr/share/common-licenses/Artistic'.
""",
}

PPKG_COPYRIGHT = \
    """This package was pypackaged(%(pv)s) by %(author)s <%(email)s> on
%(buildDate)s.

It was downloaded from %(url)s

Upstream Author: %(author)s <%(email)s>

Copyright: %(buildDateYear)s by %(author)s

License:

%(txtLicense)s

The Debian packaging is (C) %(buildDateYear)s, %(author)s <%(email)s> and
is licensed under the GPL, see above.


# Please also look if there are files or directories which have a
# different copyright/license attached and list them here.
"""

PPKG_DEBIAN_RULE = \
    """#!/usr/bin/make -f
# -*- makefile -*-
# Sample debian/rules that uses debhelper.
# This file was originally written by Joey Hess and Craig Small.
# As a special exception, when this file is copied by dh-make into a
# dh-make output file, you may use that output file without restriction.
# This special exception was added by Craig Small in version 0.37 of dh-make.

# Uncomment this to turn on verbose mode.
#export DH_VERBOSE=1

CFLAGS = -Wall -g

ifneq (,$(findstring noopt,$(DEB_BUILD_OPTIONS)))
\tCFLAGS += -O0
else
\tCFLAGS += -O2
endif

configure: configure-stamp
configure-stamp:
\tdh_testdir
\t# Add here commands to configure the package.
\ttouch configure-stamp

build: build-stamp

build-stamp: configure-stamp
\tdh_testdir
\ttouch build-stamp

clean:
\tdh_testdir
\tdh_testroot
\trm -f build-stamp configure-stamp
\tdh_clean

install: build
\tdh_testdir
\tdh_testroot
\tdh_clean -k
\tdh_installdirs

\t# ======================================================
\t#$(MAKE) DESTDIR="$(CURDIR)/debian/%(name)s" install
\tmkdir -p "$(CURDIR)/debian/%(name)s"

\t%(rules)s
\t# ======================================================

# Build architecture-independent files here.
binary-indep: build install
# We have nothing to do by default.

# Build architecture-dependent files here.
binary-arch: build install
\tdh_testdir
\tdh_testroot
\tdh_installchangelogs debian/changelog
\tdh_installdocs
\tdh_installexamples
\tdh_installman
\tdh_link
\tdh_strip
\tdh_compress
\tdh_fixperms
\tdh_installdeb
\tdh_shlibdeps
\tdh_gencontrol
\tdh_md5sums
\tdh_builddeb
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
##    Copyright (C) 2007 Khertan khertan@khertan.net
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
#
#    This file was a part of py2deb
#    This version of py2deb is a dirty hack made by Khertan
#    based on a mix of PyPackager and Original Py2Deb 0.3
#    Copyright (C) 2007 manatlan manatlan[at]gmail(dot)com


import os
import stat
import time
import tarfile
from tarfile import TarFile as _TarFile

from cStringIO import StringIO

PERMS_URW_GRW_OR = stat.S_IRUSR | stat.S_IWUSR | \
    stat.S_IRGRP | stat.S_IWGRP | \
    stat.S_IROTH

UID_ROOT = 0
GID_ROOT = 0


class TarFile(_TarFile):
    """
    """

    def addfilefromstring(self, name, theString):
        """
        """
        content = StringIO(theString)

        theFileInfo = tarfile.TarInfo(name=name)
        # Absence seems to break tgz file.
        theFileInfo.mtime = int(time.time())
        theFileInfo.size = len(content.getvalue())
        theFileInfo.mode = 3333
        self.addfile(theFileInfo, fileobj=content)


class myTarFile(object):
    """
    """

    def __init__(self, dataDirectoryPath):
        """
        """
        self._dataDirectoryPath = dataDirectoryPath

    def packed(self):
        return self._getSourcesFiles()

    def _getSourcesFiles(self):
        """
        """
        directoryPath = self._dataDirectoryPath

        outputFileObj = StringIO()  # TODO: Do more transparently?

        tarOutput = TarFile.open('sources',
                                 mode="w:gz",
                                 fileobj=outputFileObj)

        # Note: We can't use this because we need to fiddle permissions:
        #       tarOutput.add(directoryPath, arcname = "")

        # TODO: Add this as a method for TarFile and tidy-up?
        for root, dirs, files in os.walk(directoryPath):
            archiveRoot = root[len(directoryPath):]

            tarinfo = tarOutput.gettarinfo(root, archiveRoot)
            # TODO: Make configurable?
            tarinfo.uid = UID_ROOT
            tarinfo.gid = GID_ROOT
            tarinfo.uname = ""
            tarinfo.gname = ""
            tarOutput.addfile(tarinfo)

            for f in files:
                tarinfo = tarOutput.gettarinfo(os.path.join(root, f),
                                               os.path.join(archiveRoot, f))
                tarinfo.uid = UID_ROOT
                tarinfo.gid = GID_ROOT
                tarinfo.uname = ""
                tarinfo.gname = ""
                tarOutput.addfile(tarinfo, file(os.path.join(root, f)))

        tarOutput.close()

        data_tar_gz = outputFileObj.getvalue()

        return data_tar_gz



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
##    Copyright (C) 2007 Khertan khertan@khertan.net
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
#
#    This file was a part of py2deb
#    This version of py2deb is a dirty hack made by Khertan based on a mix of
#    PyPackager and Original Py2Deb 0.3
#    Copyright (C) 2007 manatlan manatlan[at]gmail(dot)com


import os
import ppkg_md5hash


class ChangesFile(object):

    """
    """
    def __init__(self, ChangedBy, description, changes,
                 files, category, repository, **kwargs):
        self.options = kwargs  # TODO: Is order important?
        self.description = description
        self.changes = changes
        self.files = files
        self.category = category
        self.repository = repository
        self.ChangedBy = ChangedBy

    def getContent(self):
        """
        """
        content = ["%s: %s" % (k, v)
                   for k, v in self.options.iteritems()]

        if self.description:
            description = self.description.replace("\n", "\n ")
            content.append('Description: ')
            content.append(' %s' % description)

        if self.changes:
            changes = self.changes.replace("\n", "\n ")
            content.append('Changes: ')
            content.append(' %s' % changes)

        if self.ChangedBy:
            content.append("Changed-By: %s" % self.ChangedBy)

        content.append('Files:')

        for onefile in self.files:
            print onefile
            md5 = ppkg_md5hash.md5sum(onefile)
            size = os.stat(onefile).st_size.__str__()
            content.append(' ' + md5 + ' ' + size + ' ' + self.category + ' '
                           + self.repository + ' ' + os.path.basename(onefile))

        print "\n".join(content) + "\n"
        return "\n".join(content) + "\n\n"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
##    Copyright (C) 2007 Khertan khertan@khertan.net
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.


import os


class PyPackagerException(Exception):
    pass


class SpecFile(object):

    def __init__(self, kwargs):
        self.options = kwargs
        self.__files = kwargs['_PyPackager__files']

    def _getContent(self):
        rules = []
        files = []
        for path in self.__files:
            for ofile, nfile in self.__files[path]:
                if os.path.isfile(ofile):
                    # it's a file
                    src_path = os.path.relpath(ofile, self.options['name'])
                    dst_path = "%{buildroot}" \
                               + os.path.join(path,
                                              os.path.dirname(nfile))
                    nname = os.path.basename(nfile)
                    files.append(os.path.join(path,
                                              os.path.dirname(nfile),
                                              nname))
                    rules.append('mkdir -p "%s"' % dst_path)

                    # make a line RULES to be sure the destination folder
                    # is created and one for copying the file
                    rules.append('cp -a "%s" "%s"' %
                                (src_path, os.path.join(dst_path, nname)))

                else:
                    raise PyPackagerException("unknown file '' " % ofile)
                    # shouldn't be raised (because controlled before)

        self.options['specrules'] = '\n'.join(rules)
        self.options['packedfiles'] = '\n'.join(files)

        content = """
Name: %(name)s
Version: %(version)s
Release: %(buildversion)s
Summary: %(summary)s
Group: %(section)s
License: %(license)s
URL: %(url)s
Source0: %(sources)s

BuildRoot: %%{_tmppath}/%%{name}-%%{version}-%%{release}-root-%%(%%{__id_u} -n)
BuildArch: noarch
BuildRequires: %(builddepends)s
Requires: %(depends)s

%%description
%(description)s

%%prep
%%setup -n %(name)s

%%build

%%install
%(specrules)s
""" % self.options
        if self.options['postinst']:
            content = content + """
%%post
%(postinst)s
""" % self.options
        if self.options['postrm']:
            content = content + """%%postun
%(postrm)s
""" % self.options
        if self.options['prerm']:
            content = content + """%%preun
%(prerm)s
""" % self.options
        if self.options['preinst']:
            content = content + """%%pre
%(preinst)s
"""

        content = content + """%%clean
rm -rf %%{buildroot}


%%files
%%defattr(-,root,root,-)
%(packedfiles)s

""" % self.options
        return content

    content = property(_getContent, doc="")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
##    Copyright (C) 2007 Khertan khertan@khertan.net
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
#
# A pure Python way of producing .deb files suitable installation
# on the Maemo platform.
# Original Author: follower@rancidbacon.com
# Date: 15 September 2006
# License: GPL 2.0


import os
import stat
import time
import tarfile
from tarfile import TarFile as _TarFile
import sys
from cStringIO import StringIO

import ppkg_arfile

FILENAME_DEB_VERSION = "debian-binary"
FILE_CONTENT_DEB_VERSION = "2.0\x0a"

FILENAME_CONTROL_TAR_GZ = "control.tar.gz"
FILENAME_DATA_TAR_GZ = "data.tar.gz"

PERMS_URW_GRW_OR = stat.S_IRUSR | stat.S_IWUSR | \
    stat.S_IRGRP | stat.S_IWGRP | \
    stat.S_IROTH

UID_ROOT = 0
GID_ROOT = 0


class ControlFile(object):
    """
    """

    def __init__(self, Icon, BugTracker, DisplayName, PreInst,
                 PostInst, PreRm, PostRm,
                 long_description="",
                 Description="",
                 UpgradeDescription=None,
                 MaemoFlags='visible',
                 MeegoDesktopEntryFilename=None,
                 createDigsigsums=False,
                 aegisManifest=None,
                 **kwargs
                 ):
        """
        """
        self.options = kwargs  # TODO: Is order important?

        # TODO: Clean-up special handling of description
        self.description = Description
        self.long_description = long_description
        self.icon = Icon
        self.bugtracker = BugTracker
        self.displayname = DisplayName
        self.preinst = PreInst
        self.postinst = PostInst
        self.prerm = PreRm
        self.postrm = PostRm
        self.upgrade_description = UpgradeDescription
        self.maemo_flags = MaemoFlags
        self.meego_desktop_entry_filename = MeegoDesktopEntryFilename
        self.createDigsigsums = createDigsigsums
        self.aegisManifest = aegisManifest

    def _getContent(self):
        """
        """
        content = ["%s: %s" % (k, v)
                   for k, v in self.options.iteritems()]

        if self.bugtracker:
            content.append("Bugtracker: %s" % self.bugtracker)
        if self.displayname:
            content.append("Maemo-Display-Name: %s" % self.displayname)
        if self.maemo_flags:
            content.append("Maemo-Flags: %s" % self.maemo_flags)
        if self.meego_desktop_entry_filename:
            content.append("Meego-Desktop-Entry-Filename: %s"
                           % self.meego_desktop_entry_filename)

        if self.description:
            self.description = self.description.replace("\n", "\n ")
            content.append("Description: %s" % self.description)

            if self.long_description:
                self.long_description = \
                    self.long_description.replace("\n", "\n ")
                content.append(" " + self.long_description)

        if self.upgrade_description:
            self.upgrade_description = \
                self.upgrade_description.rstrip('\n').replace("\n", "\n ")
            content.append("Maemo-Upgrade-Description: %s"
                           % self.upgrade_description)

        if self.icon:
            content.append("Maemo-Icon-26: %s" % self.icon)

        print "\n".join(content) + "\n"
        return "\n".join(content) + "\n"

    content = property(_getContent, doc="")


class TarFile(_TarFile):
    """
    """

    def addfilefromstring(self, name, theString):
        """
        """
        content = StringIO(theString)

        theFileInfo = tarfile.TarInfo(name=name)
        # Absence seems to break tgz file
        theFileInfo.mtime = int(time.time())
        theFileInfo.size = len(content.getvalue())
        theFileInfo.uid = UID_ROOT
        theFileInfo.gid = GID_ROOT
        theFileInfo.mode = 3333

        self.addfile(theFileInfo, fileobj=content)


class MaemoPackage(object):
    """
    """

    def __init__(self, controlFile, files):
        """
        """
        self.controlFile = controlFile
        self.__files = files

    def packed(self):
        """
        """
        theDeb = ppkg_arfile.ArFile()

        ## Add the debian package version
        theDeb.files.append(self._getVersionFile())

        ## Add the compressed control related file(s)
        theDeb.files.append(self._getControlFiles())

        ## Add compressed data file(s)
        theDeb.files.append(self._getDataFiles())

        ## Add _aegis if needed
        if self.controlFile.aegisManifest:
            theDeb.files.append(self._getAegisFile())

        return theDeb.packed()

    def _getSize(self):
        size = 0
        paths = self.__files.keys()
        paths.sort()
        CURRENT = os.path.dirname(sys.argv[0])
        for path in paths:
            for pfile, nfile in self.__files[path]:
                size = size + (getattr(os.stat(
                    os.path.join(CURRENT, pfile)),
                    'st_size') / 1024)
        return size

    def _getVersionFile(self):
        """
        """
        debVersionFile = \
            ppkg_arfile.FileInfo(name=FILENAME_DEB_VERSION,
                                 modificationTime=int(time.time()),
                                 userId=UID_ROOT,
                                 groupId=GID_ROOT,
                                 fileMode=PERMS_URW_GRW_OR,
                                 fileSize=len(FILE_CONTENT_DEB_VERSION),
                                 data=FILE_CONTENT_DEB_VERSION)

        return debVersionFile

    def _getControlFiles(self):
        """
        """
        debControlFile = self.controlFile.content \
            + 'Installed-Size: ' + str(self._getSize()) + '\n'

        outputFileObj = StringIO()
        tarOutput = TarFile.open(FILENAME_CONTROL_TAR_GZ,
                                 mode="w:gz",
                                 fileobj=outputFileObj)

        tarOutput.addfilefromstring("control", debControlFile)
        if (self.controlFile.preinst):
            tarOutput.addfilefromstring("preinst", self.controlFile.preinst)
        if (self.controlFile.postinst):
            tarOutput.addfilefromstring("postinst", self.controlFile.postinst)
        if (self.controlFile.prerm):
            tarOutput.addfilefromstring("prerm", self.controlFile.prerm)
        if (self.controlFile.postrm):
            tarOutput.addfilefromstring("postrm", self.controlFile.postrm)

        # TODO: Add `postinst` here if needed.
        if self.controlFile.createDigsigsums:
            from ppkg_digsigsums import generate_digsigsums, hash_file
            package_name = self.controlFile.options['Package']
            digsigsums = generate_digsigsums(package_name, self.__files)
            if (self.controlFile.preinst):
                digsigsums = digsigsums + \
                    hash_file(package_name, self.controlFile.preinst,
                              'var/lib/dpkg/info/'
                              + package_name + '.preinst')
            if (self.controlFile.postinst):
                digsigsums = digsigsums + \
                    hash_file(package_name,
                              self.controlFile.postinst,
                              'var/lib/dpkg/info/'
                              + package_name + '.postinst')
            if (self.controlFile.prerm):
                digsigsums = digsigsums + \
                    hash_file(package_name,
                              self.controlFile.prerm,
                              'var/lib/dpkg/info/'
                              + package_name + '.prerm')
            if (self.controlFile.postrm):
                digsigsums = digsigsums + \
                    hash_file(package_name,
                              self.controlFile.postrm,
                              'var/lib/dpkg/info/'
                              + package_name + '.postrm')

            tarOutput.addfilefromstring("digsigsums", digsigsums)
            print digsigsums

        if self.controlFile.aegisManifest:
            tarOutput.addfilefromstring("%s.aegis" % FILENAME_DEB_VERSION,
                                        self.controlFile.aegisManifest)

        tarOutput.close()

        control_tar_gz = outputFileObj.getvalue()

        controlFile = ppkg_arfile.FileInfo(name=FILENAME_CONTROL_TAR_GZ,
                                           modificationTime=int(time.time()),
                                           userId=UID_ROOT,
                                           groupId=GID_ROOT,
                                           fileMode=PERMS_URW_GRW_OR,
                                           fileSize=len(control_tar_gz),
                                           data=control_tar_gz)

        return controlFile

    def _getAegisFile(self):
        return \
            ppkg_arfile.FileInfo(name=FILENAME_DEB_VERSION,
                                 modificationTime=int(time.time()),
                                 userId=UID_ROOT,
                                 groupId=GID_ROOT,
                                 fileMode=PERMS_URW_GRW_OR,
                                 fileSize=len(self.controlFile.aegisManifest),
                                 data=self.controlFile.aegisManifest)

    def _getDataFiles(self):
        """
        """

        outputFileObj = StringIO()

        tarOutput = TarFile.open(FILENAME_DATA_TAR_GZ,
                                 mode="w:gz",
                                 fileobj=outputFileObj)

        # Note: We can't use this because we need to fiddle permissions:
        #       tarOutput.add(directoryPath, arcname = "")

        # TODO: Add this as a method for TarFile and tidy-up?
        paths = self.__files.keys()
        paths.sort()

        CURRENT = sys.path[0]
        for path in paths:
            tarinfo = tarOutput.gettarinfo(os.path.join(CURRENT), path)
            tarinfo.uid = UID_ROOT
            tarinfo.gid = GID_ROOT
            tarinfo.uname = ""
            tarinfo.gname = ""
            tarOutput.addfile(tarinfo)

            #We need to get all folder and create them before
            folders = []
            for pfile, nfile in self.__files[path]:
                dirpath = os.path.dirname(pfile)
                while dirpath:
                    if dirpath not in folders:
                        folders.append(dirpath)
                    dirpath = os.path.dirname(dirpath)

            folders = list(set(folders))
            folders.sort()
            print folders

            for folder in folders:
                if folder.endswith('/'):
                    folder = folder[:0]
                if folder.startswith('/'):
                    folder = folder[1:]
                tarinfo = tarOutput.gettarinfo(os.path.join(
                                               CURRENT, folder),
                                               os.path.join(path, folder))
                tarinfo.uid = UID_ROOT
                tarinfo.gid = GID_ROOT
                tarinfo.uname = ""
                tarinfo.gname = ""
                tarinfo.mtime = int(time.time())
                tarOutput.addfile(tarinfo)

            for pfile, nfile in self.__files[path]:
                rfile = os.path.normpath(os.path.join(path, nfile))
                if rfile.startswith('/'):
                    rfile = rfile[1:]
                if not rfile in folders:
                    tarOutput.addfilefromstring(
                        rfile,
                        file(os.path.join(CURRENT, pfile)).read())

        tarOutput.close()

        data_tar_gz = outputFileObj.getvalue()

        dataFile = ppkg_arfile.FileInfo(name=FILENAME_DATA_TAR_GZ,
                                        modificationTime=int(time.time()),
                                        userId=UID_ROOT,
                                        groupId=GID_ROOT,
                                        fileMode=PERMS_URW_GRW_OR,
                                        fileSize=len(data_tar_gz),
                                        data=data_tar_gz)
        return dataFile



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
##    Copyright (C) 2007 Khertan khertan@khertan.net
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
#
#
# Author: follower@rancidbacon.com
#
# Date: 13 September 2006
#
# License: GPL 2.0
#
# Note:
#
#  * This has been written based on the format described in the AR(5) man page.
#
#  * This module does not fully implement the file format as described--there
#    is no support for file names longer than 16 characters--nor is there
#    any error checking that this does not occur.
#
#  * There is no functionality for reading `ar` format files--only writing for
#    them.
#
#  * The motivation for writing this was for creating a pure-Python way
#    of constructing ".deb" files--which is an archive file with 3 specific
#    files in it--thus a limited functionality requirement.
#
import os

MAGIC_STRING = "!<arch>\n"


def paddedString(text, length):
    """

    Return a string with (possibly no) trailing spaces so the total
    length of the string is `length` characters.

    """
    return (text + length * " ")[:length]


NAME_BUFFER_LENGTH = 16
MOD_TIME_BUFFER_LENGTH = 12
ID_BUFFER_LENGTH = 6
FILE_MODE_BUFFER_LENGTH = 8
FILE_SIZE_BUFFER_LENGTH = 10

NO_PADDING_CHAR = ""
PADDING_CHAR = "\n"

FILE_HEADER_TRAILER = "`\n"


class FileInfo(object):
    """

    Represents a single file in the archive.

    The name is a little of a misnomer as it also (currently) holds the
    file content.

    """

    def __init__(self,
                 name,
                 modificationTime,
                 userId, groupId,
                 fileMode,
                 fileSize,
                 data):
        """
        """
        self.name = name  # TODO: Ensure ar-compatible
        self.modificationTime = modificationTime
        self.userId = userId
        self.groupId = groupId
        self.fileMode = fileMode
        self.fileSize = fileSize
        self.data = data

    def _getHeader(self):
        """

        Returns the header for the file as documented in AR(5).

        """
        data = []

        data.append(paddedString(self.name, NAME_BUFFER_LENGTH))

        data.append(paddedString(str(self.modificationTime),
                                 MOD_TIME_BUFFER_LENGTH))

        data.append(paddedString(str(self.userId), ID_BUFFER_LENGTH))

        data.append(paddedString(str(self.groupId), ID_BUFFER_LENGTH))

        data.append(paddedString("%o" % self.fileMode,
                                 FILE_MODE_BUFFER_LENGTH))

        data.append(paddedString(str(self.fileSize), FILE_SIZE_BUFFER_LENGTH))

        data.append(FILE_HEADER_TRAILER)

        return "".join(data)

    header = property(_getHeader, doc="")

    def packed(self):
        """

        Returns the "packed" form suitable for appended to an `ar` file.

        The packing concatenates the header, data and--if
        necessary--trailing padding. The padding is required to obey
        the "Objects in the archive are always an even number of bytes
        long;" requirement of the file format.

        """
        return self.header + \
            self.data + \
            [NO_PADDING_CHAR, PADDING_CHAR][self.fileSize % 2]  # Pad odd size

    def _fromFilePath(filepath):
        """
        """
        statResult = os.stat(filepath)
        return FileInfo(name=os.path.basename(filepath),
                        modificationTime=statResult.st_mtime,
                        userId=statResult.st_uid,
                        groupId=statResult.st_gid,
                        fileMode=statResult.st_mode,
                        fileSize=statResult.st_size,
                        data=open(filepath, "rb").read())  # Hardly efficient!

    fromFilePath = staticmethod(_fromFilePath)


class ArFile(object):
    """

    Represents a complete archive.

    Files can be added by appending the paths to the `files` list.

    """

    def __init__(self):
        """
        """
        self.files = []  # Can be a string file path or a FileInfo instance.

    def packed(self):
        """

        Returns the "packed" form of the archive--suitable for writing
        to disc.

        """
        content = []

        content.append(MAGIC_STRING)

        for theFile in self.files:
            if isinstance(theFile, FileInfo):
                packedFile = theFile.packed()
            else:
                packedFile = FileInfo.fromFilePath(theFile).packed()

            content.append(packedFile)

        return "".join(content)


if __name__ == "__main__":
    import sys

    if len(sys.argv) < 2:
        print "Usage: %s  <filepath> [...]" % sys.argv[0]
        raise SystemExit

    #print FileInfo.fromFilePath(filepath).header
    #print FileInfo.fromFilePath(filepath).packed()

    archiveFile = ArFile()

    for theFilepath in sys.argv[1:]:
        archiveFile.files.append(theFilepath)

    sys.stdout.write(archiveFile.packed())



import os

from setuptools import setup
import pypackager

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "pypackager",
    version = pypackager.__version__,
    author = "Benoit HERVIER",
    author_email = "khertan@khertan.net",
    description = ("PyPackager is an onboard developpers tools"+
                   "which is usefull to create sources and binary"+
                   "Maemo/Harmattan/MeeGo package."),
    license = "GPL",
    keywords = "package debian maemo",
    url = "http://khertan.net/pypackager",
    packages= ['pypackager'],
    long_description=read('README.md'),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Operating System :: POSIX :: Linux",
        "Intended Audience :: Developers",]
,)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
##    Copyright (C) 2007 Khertan khertan@khertan.net
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## Please wear sunglasses, else it could hurt your eyes

import os
import shutil
import time
from glob import glob
from datetime import datetime

__version__ = '4.0.0'
__build__ = '1'
__author__ = "khertan"
__mail__ = "khertan@khertan.net"
__changelog__ = '''* 3.6.0: Fix permission on script post/pre inst/rm."
* 3.7.0 : Add experimental specfile creation
* 3.7.1 : fix incorrect generation of the spec file
* 3.7.2 : fix error in spec file
* 3.7.3 : add possibilities to have different dependancies for spec / deb
* 4.0.0 : Put builded packages in different sub folder as changes file for
          debian and rpm source have same name but different structure.
          Cleaning code'''


class PyPackagerException(Exception):
    pass


class PyPackager(object):

    def __setitem__(self, path, files):
        if not type(files) == list:
            raise PyPackagerException("value of key path '%s' is not a list"
                                      % path)
        if not files:
            raise PyPackagerException("value of key path '%s'"
                                      " should'nt be empty" % path)
        if not path.startswith("/"):
            raise PyPackagerException("key path '%s' malformed"
                                      " (don't start with '/')" % path)
        if path.endswith("/"):
            raise PyPackagerException("key path '%s' malformed"
                                      " (shouldn't ends with '/')" % path)
        nfiles = []
        for file in files:
            if ".." in file:
                raise PyPackagerException("file '%s' contains"
                                          " '..', please avoid that!" % file)
            if "|" in file:
                if file.count("|") != 1:
                    raise PyPackagerException("file '%s' is incorrect"
                                              " (more than one pipe)" % file)
                file, nfile = file.split("|")
            else:
                nfile = file  # same localisation
            if os.path.isdir(file):
                raise PyPackagerException("file '%s' is a folder,"
                                          " and PyPackager refuse folders !"
                                          % file)
            if not os.path.isfile(file):
                raise PyPackagerException("file '%s' doesn't exist" % file)
            if file.startswith("/"):    # if an absolute file is defined
                if file == nfile:         # and not renamed (pipe trick)
                    nfile = os.path.basename(file)  # it's simply
                                                    #copied to 'path'
            nfiles.append((file, nfile))
        #sort according new name (nfile)
        nfiles.sort(lambda a, b: cmp(a[1], b[1]))
        self.__files[path] = nfiles

    def __delitem__(self, k):
        del self.__files[k]

    def __init__(self, name):
        self.name = name
        self.display_name = name
        self.summary = 'No summary'
        self.description = "No description"
        self.license = "gpl"
        self.depends = "python"
        self.rpm_depends = None
        self.suggests = ""
        self.section = "user/other"
        self.arch = ""

        self.author = ""
        self.maintainer = ""
        self.email = ""
        self.version = "0.0.0"
        self.buildversion = '1'
        self.__files = {}

        self.preinst = None
        self.postinst = None
        self.prerm = None
        self.postrm = None

        self.changelog = None

        self.url = "http://"
        self.bugtracker = ""
        self.icon = None

        #Maemo_Flags (harmattan)
        self.maemo_flags = ''
        #MeeGo_Desktop_Entry_Filename (harmattan)
        self.meego_desktop_entry_filename = ''

        #Aegis Digsigsums
        self.createDigsigsums = False
        #Aegis Manifest
        self.aegisManifest = ''

    def __repr__(self):
        from ppkg_constants import PPKG_RETURN
        paths = self.__files.keys()
        paths.sort()
        files = []
        for path in paths:
            for afile, nfile in self.__files[path]:
                rfile = os.path.join(path, nfile)
                if nfile == afile:
                    files.append(rfile)
                else:
                    files.append(rfile + " (%s)" % afile)
        files.sort()
        self.files = "\n".join(files)

        lscripts = [self.preinst and "preinst",
                    self.postinst and "postinst",
                    self.prerm and "prerm",
                    self.postrm and "postrm", ]
        self.scripts = lscripts and \
            ", ".join([i for i in lscripts if i]) or "None"
        return PPKG_RETURN % self.__dict__

    def _create_dist_folder(self):
        if not os.path.exists('dists'):
            os.mkdir('dists')
        self._dist_dir = os.path.join(os.path.join('dists',
                                                   self.version
                                                   + '-'
                                                   + self.buildversion))
        if not os.path.exists(self._dist_dir):
            os.mkdir(self._dist_dir)

    def generate(self, options):
        self._create_dist_folder()

        if 'debian_binary' in options:
            self.generate_debian_binary()
        if 'debian_source' in options:
            self.generate_debian_source()
        if 'rpm_source' in options:
            self.generate_rpm_source()

    def generate_debian_binary(self):
        from ppkg_debfile import MaemoPackage, ControlFile
        import base64

        #Create debian binary dist folder
        _dist_dir = os.path.join(self._dist_dir, 'debian_binary')
        if not os.path.exists(_dist_dir):
            os.mkdir(_dist_dir)

        try:
            iconb64 = "".join(base64.encodestring(
                open(self.icon).read()).split("\n")[0:-1])
        except:
            iconb64 = ''

        theMaemoPackage = MaemoPackage(
            ControlFile(Icon=str(iconb64),
                        BugTracker=self.bugtracker,
                        DisplayName=self.display_name,
                        PreInst=self.preinst,
                        PostInst=self.postinst,
                        PreRm=self.prerm,
                        PostRm=self.postrm,
                        Package=self.name,
                        Version=self.version + '-' + self.buildversion,
                        Section=self.section,
                        Priority='low',
                        Architecture=self.arch,
                        Maintainer=self.maintainer,
                        Depends=self.depends,
                        Suggest=self.suggests,
                        Description=self.description,
                        UpgradeDescription=self.upgrade_description,
                        MaemoFlags=self.maemo_flags,
                        MeegoDesktopEntryFilename=
                        self.meego_desktop_entry_filename,
                        createDigsigsums=self.createDigsigsums,
                        aegisManifest=self.aegisManifest),
            self.__files)

        open(os.path.join(_dist_dir, self.name
                          + '_'
                          + self.version
                          + '-' + self.buildversion
                          + '_' + self.arch
                          + '.deb'), "wb").write(theMaemoPackage.packed())

        #Dsc
        from ppkg_dscfile import DscFile
        import locale
        try:
            old_locale, iso = locale.getlocale(locale.LC_TIME)
            locale.setlocale(locale.LC_TIME, 'en_US')
        except:
            pass

        print os.path.join(_dist_dir, "%(name)s_%(version)s-"
                           "%(buildversion)s_%(arch)s.deb" % self.__dict__)
        dsccontent = DscFile("%(version)s-%(buildversion)s" % self.__dict__,
                             "%(depends)s" % self.__dict__,
                             (os.path.join(_dist_dir, "%(name)s_%(version)s-"
                             "%(buildversion)s_%(arch)s.deb"
                             % self.__dict__), ),
                             Format='1.0',
                             Source="%(name)s" % self.__dict__,
                             Version="%(version)s-%(buildversion)s"
                             % self.__dict__,
                             Maintainer="%(maintainer)s <%(email)s>"
                             % self.__dict__,
                             Architecture="%(arch)s" % self.__dict__,)
        f = open(os.path.join(_dist_dir,
                              "%(name)s_%(version)s-%(buildversion)s.dsc"
                              % self.__dict__), "wb")
        f.write(dsccontent._getContent())
        f.close()

        #Changes
        from ppkg_changesfile import ChangesFile
        changescontent = ChangesFile(
            "%(author)s <%(email)s>" % self.__dict__,
            "%(description)s" % self.__dict__,
            "%(changelog)s" % self.__dict__,
            (os.path.join(_dist_dir,
                          "%(name)s_%(version)s-%(buildversion)s_%(arch)s.deb"
             % self.__dict__,),
             os.path.join(_dist_dir,
                          "%(name)s_%(version)s-%(buildversion)s.dsc"
             % self.__dict__, ),),
            "%(section)s" % self.__dict__,
            "%(repository)s" % self.__dict__,
            Format='1.7',
            Date=time.strftime("%a, %d %b %Y %H:%M:%S +0000",
                               time.gmtime()),
            Source="%(name)s" % self.__dict__,
            Architecture="%(arch)s" % self.__dict__,
            Version="%(version)s-%(buildversion)s" % self.__dict__,
            Distribution="%(distribution)s" % self.__dict__,
            Urgency="%(urgency)s" % self.__dict__,
            Maintainer="%(maintainer)s <%(email)s>" % self.__dict__)
        f = open(os.path.join(_dist_dir,
                              "%(name)s_%(version)s-%(buildversion)s.changes"
                              % self.__dict__), "wb")
        f.write(changescontent.getContent())
        f.close()
        try:
            locale.setlocale(locale.LC_TIME, old_locale)
        except:
            pass

    def generate_common_source(self, _dist_dir):
        from ppkg_constants import PPKG_SECTIONS, PPKG_ARCHS, PPKG_LICENSES, \
            PPKG_DEBIAN_CHANGESLOG, PPKG_DEBIAN_CONTROL, PPKG_LICENSES_TEXT, \
            PPKG_COPYRIGHT, PPKG_DEBIAN_RULE

        if not sum([len(i) for i in self.__files.values()]) > 0:
            raise PyPackagerException("no files are defined")

        if not self.changelog:
            self.changelog = "  * no changelog"

        if self.section not in PPKG_SECTIONS:
            raise PyPackagerException("section '%s' is unknown (%s)"
                                      % (self.section,
                                      str(PyPackager.SECTIONS)))

        if self.arch not in PPKG_ARCHS:
            raise PyPackagerException("arch '%s' is unknown (%s)"
                                      % (self.arch,
                                      str(PyPackager.ARCHS)))

        if self.license not in PPKG_LICENSES:
            raise PyPackagerException("License '%s' is unknown (%s)"
                                      % (license,
                                      str(PyPackager.LICENSES)))

        # create dates (buildDate,buildDateYear)
        d = datetime.now()
        self.buildDate = d.strftime("%a, %d %b %Y %H:%M:%S +0000")
        self.buildDateYear = str(d.year)

        #clean description (add a space before each next lines)
        self.description = self.description.replace("\r", "").strip()
        self.description = "\n ".join(self.description.split("\n"))

        #clean upgrade_description (add a space before each next lines)
        self.upgrade_description = \
            self.upgrade_description.replace("\r", "").strip()
        self.upgrade_description = \
            "\n ".join(self.upgrade_description.split("\n"))

        #Really crappy
        self.changelog = self.changelog.replace('\r', '').strip()
        changeslog = ''
        for line in self.changelog.split('\n'):
            changeslog = changeslog + '  ' + line + '\n'
        self.changelog = changeslog

        self.TEMP = ".pypackager_build_folder"
        DEST = os.path.join(self.TEMP, self.name)
        DEBIAN = os.path.join(DEST, "debian")

        # let's start the process
        try:
            shutil.rmtree(self.TEMP)
        except:
            pass

        os.makedirs(DEBIAN)
        try:
            rules = []
            dirs = []
            for path in self.__files:
                for ofile, nfile in self.__files[path]:
                    if os.path.isfile(ofile):
                        # it's a file

                        if ofile.startswith("/"):  # if absolute path
                            # we need to change dest
                            dest = os.path.join(DEST, nfile)
                        else:
                            dest = os.path.join(DEST, ofile)

                        # copy file to be packaged
                        destDir = os.path.dirname(dest)
                        if not os.path.isdir(destDir):
                            os.makedirs(destDir)

                        shutil.copy2(ofile, dest)

                        ndir = os.path.join(path, os.path.dirname(nfile))
                        nname = os.path.basename(nfile)

                        # make a line RULES to be sure the destination folder
                        # and one for copying the file is created
                        fpath = "/".join(["$(CURDIR)", "debian",
                                self.name + ndir])
                        rules.append('mkdir -p "%s"' % fpath)
                        rules.append('cp -a "%s" "%s"'
                                     % (ofile, os.path.join(fpath, nname)))

                        # append a dir
                        dirs.append(ndir)

                    else:
                        # shouldn't be raised (because controlled before)
                        raise PyPackagerException("unknown file '' " % ofile)

            # make rules right
            self.rules = "\n\t".join(rules) + "\n"

            # make dirs right
            dirs = [i[1:] for i in set(dirs)]
            dirs.sort()

            #===================================================
            # CREATE debian/dirs
            #===================================================
            open(os.path.join(DEBIAN, "dirs"), "wb").write("\n".join(dirs))

            #===================================================
            # CREATE debian/changelog
            #===================================================
            clog = PPKG_DEBIAN_CHANGESLOG % self.__dict__

            open(os.path.join(DEBIAN, "changelog"), "wb").write(clog)

            #===================================================
            #Create pre/post install/remove
            #===================================================
            def mkscript(name, dest):
                if name and name.strip() != "":
                    if (os.path.isfile(name)):
                        content = file(name).read()
                    else:   # it's a script
                        content = name
                    print os.path.join(DEBIAN, dest)
                    open(os.path.join(DEBIAN, dest), "wb").write(content)

            mkscript(self.preinst, "preinst")
            mkscript(self.postinst, "postinst")
            mkscript(self.prerm, "prerm")
            mkscript(self.postrm, "postrm")

            #===================================================
            # CREATE debian/compat
            #===================================================
            open(os.path.join(DEBIAN, "compat"), "w").write("7\n")

            #===================================================
            # CREATE icon
            #===================================================
            self.iconstr = ""
            if self.icon is not None and os.path.exists(self.icon):
                try:
                    import base64
                    iconb64 = "".join(base64.encodestring(
                        open(self.icon).read()).split("\n")[0:-1])
                    self.iconstr = "XB-Maemo-Icon-26: %s" % (iconb64)
                except:
                    pass

            #===================================================
            # CREATE bugtracker
            #===================================================
            self.bugtrackerstr = "XSBC-Bugtracker: %s" % (self.bugtracker)

            self.build_depends = 'debhelper (>= 8.0.0)'
            self.build_depends = self.build_depends \
                + ", pkg-config, aegis-builder" \
                if (self.aegisManifest) else ''

            #===================================================
            # CREATE debian/control
            #===================================================
            txt = PPKG_DEBIAN_CONTROL % self.__dict__

            open(os.path.join(DEBIAN, "control"), "wb").write(txt)

            #===================================================
            # CREATE debian/copyright
            #===================================================

            self.txtLicense = PPKG_LICENSES_TEXT[self.license]
            self.pv = __version__
            txt = PPKG_COPYRIGHT % self.__dict__
            open(os.path.join(DEBIAN, "copyright"), "wb").write(txt)

            #===================================================
            # CREATE debian/rules
            #===================================================
            txt = PPKG_DEBIAN_RULE % self.__dict__
            if self.aegisManifest:
                txt = txt + ('\taegis-deb-add -control debian/%(name)s/'
                             'DEBIAN/control ..'
                             ' debian/%(name)s.aegis=_aegis' % self.__dict__)
            txt = txt + """
binary: binary-indep binary-arch
.PHONY: build clean binary-indep binary-arch binary install configure
""" % self.__dict__
            open(os.path.join(DEBIAN, "rules"), "wb").write(txt)
            os.chmod(os.path.join(DEBIAN, "rules"), 0755)

            #===================================================
            # CREATE debian/digsigsums
            #===================================================
            if self.createDigsigsums:
                from ppkg_digsigsums import generate_digsigsums
                open(os.path.join(DEBIAN, "digsigsums"), "wb") \
                    .write(generate_digsigsums(self.name, self.__files))

            #===================================================
            # CREATE debian/_aegis manifest
            # <aegis name="...">
            #  <provide> ... </provide>
            #  <constraint> ... </constraint>
            #  <account> ... </account>
            #  <request> ... </request>
            #  <domain> ... </domain>
            #  <docstring> ... </docstring>
            #</aegis>'''
            #===================================================

            if self.aegisManifest:
                mkscript(self.aegisManifest, '%s.aegis'
                         % self.__dict__['name'])

            #Tar
            from ppkg_tarfile import myTarFile
            tarcontent = myTarFile("%(DEST)s" % locals())
            open("%(TEMP)s/%(name)s_%(version)s-%(buildversion)s.tar.gz"
                 % self.__dict__, "wb").write(tarcontent.packed())

            l = glob("%(TEMP)s/%(name)s*.tar.gz" % self.__dict__)
            if len(l) != 1:
                raise PyPackagerException("don't find source package tar.gz")
            tar = os.path.join(_dist_dir, os.path.basename(l[0]))
            shutil.move(l[0], tar)
        except:
            return False
        return True

    def generate_rpm_source(self):
        _dist_dir = os.path.join(self._dist_dir, 'rpm_source')
        if not os.path.exists(_dist_dir):
            os.mkdir(_dist_dir)
        if self.generate_common_source(_dist_dir):

            #Specs
            from ppkg_specfile import SpecFile
            import locale
            try:
                old_locale, iso = locale.getlocale(locale.LC_TIME)
                locale.setlocale(locale.LC_TIME, 'en_US')
            except:
                pass

            #Write spec
            self.builddepends = 'python-devel'
            if self.rpm_depends is None:
                self.rpm_depends = self.depends
            else:
                self.depends = self.rpm_depends
            self.sources = os.path.join(_dist_dir,
                                        "%(name)s_%(version)s-"
                                        "%(buildversion)s.tar.gz"
                                        % self.__dict__)
            specfile = SpecFile(self.__dict__)
            f = open(os.path.join(_dist_dir,
                                  "%(name)s_%(version)s"
                                  "-%(buildversion)s.spec"
                                  % self.__dict__), "wb")

            #Write changes
            changeslog = ('* ' + time.strftime("%a %b %d %Y", time.gmtime())
                          + (" %(author)s <%(email)s> - "
                             "%(version)s-%(buildversion)s\n")
                          % self.__dict__)
            for index, line in enumerate(self.changelog.split('\n')):
                if line.startswith('-'):
                    changeslog = changeslog + line + '\n'
                else:
                    changeslog = changeslog + '- %s\n' % line

            f.write(specfile.content)
            f = open(os.path.join(_dist_dir,
                                  "%(name)s_%(version)s"
                                  "-%(buildversion)s.changes"
                                  % self.__dict__), "wb")
            f.write(changeslog)
            f.close()

            try:
                locale.setlocale(locale.LC_TIME, old_locale)
            except:
                pass

            shutil.rmtree(self.TEMP)

    def generate_debian_source(self):
        _dist_dir = os.path.join(self._dist_dir, 'debian_source')
        if not os.path.exists(_dist_dir):
            os.mkdir(_dist_dir)
        if self.generate_common_source(_dist_dir):

            #Dsc
            from ppkg_dscfile import DscFile
            import locale
            try:
                old_locale, iso = locale.getlocale(locale.LC_TIME)
                locale.setlocale(locale.LC_TIME, 'en_US')
            except:
                pass
            dsccontent = DscFile('%(version)s-%(buildversion)s'
                                 % self.__dict__,
                                 self.build_depends,
                                 (os.path.join(_dist_dir,
                                               '%(name)s_%(version)s'
                                               '-%(buildversion)s.tar.gz'
                                               % self.__dict__, ),),
                                 Format='1.0',
                                 Source="%(name)s" % self.__dict__,
                                 Version="%(version)s-%(buildversion)s"
                                 % self.__dict__,
                                 Maintainer="%(maintainer)s <%(email)s>"
                                 % self.__dict__,
                                 Architecture="%(arch)s" % self.__dict__, )

            f = open(os.path.join(_dist_dir,
                                  "%(name)s_%(version)s-%(buildversion)s.dsc"
                                  % self.__dict__), "wb")
            f.write(dsccontent._getContent())
            f.close()

            #Changes
            from ppkg_changesfile import ChangesFile
            changescontent = ChangesFile(
                "%(author)s <%(email)s>" % self.__dict__,
                "%(description)s" % self.__dict__,
                "%(changelog)s" % self.__dict__,
                (
                    os.path.join(_dist_dir,
                                 "%(name)s_%(version)s-%(buildversion)s.tar.gz"
                                 % self.__dict__,),
                    os.path.join(_dist_dir,
                                 "%(name)s_%(version)s-%(buildversion)s.dsc"
                                 % self.__dict__,),
                ),
                "%(section)s" % self.__dict__,
                "%(repository)s" % self.__dict__,
                Format='1.7',
                Date=time.strftime("%a, %d %b %Y %H:%M:%S +0000",
                                   time.gmtime()),
                Source="%(name)s" % self.__dict__,
                Architecture="%(arch)s" % self.__dict__,
                Version="%(version)s-%(buildversion)s"
                % self.__dict__,
                Distribution="%(distribution)s" % self.__dict__,
                Urgency="%(urgency)s" % self.__dict__,
                Maintainer="%(maintainer)s <%(email)s>" % self.__dict__)
            f = open(os.path.join(_dist_dir,
                                  "%(name)s_%(version)s-"
                                  "%(buildversion)s.changes"
                                  % self.__dict__), "wb")
            f.write(changescontent.getContent())
            f.close()

            try:
                locale.setlocale(locale.LC_TIME, old_locale)
            except:
                pass

            shutil.rmtree(self.TEMP)

if __name__ == "__main__":
    print 'Look at make.py in the git repository to see how to use it'




#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
##    Copyright (C) 2007 Khertan khertan@khertan.net
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

#Original code was picked from MeeGo.com forum. Author is merlin1991

import sys
import os
import hashlib


def check_file(filePath):
    if not os.path.exists(filePath):  # To be able to hash StringIO
        return True
    fh = open(filePath, 'r')
    if fh.read(2) == '#!':
        fh.close()
        return True
    fh.close()
    return False


def hash_file(packageName, src_filepath, dst_filepath):
    # digsigsums format:
    # S 15 com.nokia.maemo H 40 xxxxxxxxxxxxxxxxxxxxx R 5 abcde
    # Source (might change) hash (sha1) relative path
    if 'DEBIAN/' in dst_filepath:
        dst_filepath = 'var/lib/dpkg/info/{0}.{1}'.format(packageName,
                       dst_filepath[8 + len(packageName):])
    if dst_filepath.startswith('/'):
        dst_filepath = dst_filepath[1:]
    if os.path.exists(src_filepath):
        fileToHash = open(src_filepath, 'rb')
        sha1 = hashlib.sha1()
        sha1.update(fileToHash.read())
        hashString = sha1.hexdigest()
        fileToHash.close()
    else:
        sha1 = hashlib.sha1()
        sha1.update(src_filepath)
        hashString = sha1.hexdigest()
    return 'S 15 com.nokia.maemo H 40 {0} R {1} {2}\n' \
           .format(hashString,
                   len(dst_filepath),
                   dst_filepath)


def generate_digsigsums(packageName, files):
    result = ''
    for path in files.keys():
        for pfile, nfile in files[path]:
            dst_filename = os.path.join(path, nfile)
            src_filename = os.path.join(sys.path[0], pfile)
            if check_file(src_filename):
                result = result + (hash_file(packageName,
                                             src_filename,
                                             dst_filename))
    return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
##    Copyright (C) 2007 Khertan khertan@khertan.net
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
#
# md5hash
#
# 2004-01-30
#
# Nick Vargish
#
# Simple md5 hash utility for generating md5 checksums of files.
#
# usage: md5hash <filename> [..]
#
# Use '-' as filename to sum standard input.

import md5
import sys


def sumfile(fobj):
    '''Returns an md5 hash for an object with read() method.'''
    m = md5.new()
    while True:
        d = fobj.read(8096)
        if not d:
            break
        m.update(d)
    return m.hexdigest()


def md5sum(fname):
    '''Returns an md5 hash for file fname, or stdin if fname is "-".'''
    if fname == '-':
        ret = sumfile(sys.stdin)
    else:
        try:
            f = file(fname, 'rb')
        except:
            return 'Failed to open file'
        ret = sumfile(f)
        f.close()
    return ret


# if invoked on command line, print md5 hashes of specified files.
if __name__ == '__main__':
    for fname in sys.argv[1:]:
        print '%32s  %s' % (md5sum(fname), fname)
